% This script is based on:
% Jarocinski, Karadi, Deconstructing Monetary Policy Surprises - The Role 
% of Information Shocks, forthcoming in the AEJ:Macroeconomics.
%
% The script estimates a monthly VAR with high-frequency monetary policy
% surprises and identifies shocks with cholesky.

% This script is able to estimate : 

% - A VAR without surprises, with MP shocks identified via Cholesky
% - A VAR with iid surprises, with IRFs to surprises via Cholesky given the
% result in Plagborg-Moller and Wolf (2020). 
% - Same as above but adding one additional variable at a time to the
% benchmark VAR.

clear all, close all

% Set the random seed to fix randomness across runs 

rng(72256,'twister');

%%%%%%%%%% Options %%%%%%%%%%

% SAMPLE AND DATA: spl, modname, addvar

% spl = [2003 6; 2019 12];    % Sample for Sh_CHSWP2
% spl = [2001 8; 2019 12];  % Benchmark sample 

% spl = [2001 9; 2019 12];  % Use for EEE_INFLATION_IN_11_MONTHS
% spl = [2010 1; 2019 12];  % Use for EOF_INFLATION_12_MONTHS
% spl = [2003 3; 2019 12];  % Use for SOV_BONDS_RATE_PESOS_1Y (2 obs missing - 2018:6-7) [will take about 10 minutes]
% spl = [2002 9; 2019 12];    % Use for SOV_BONDS_RATE_PESOS_2Y 
% spl = [2005 8; 2019 12];    % Use for SOV_BONDS_RATE_PESOS_5Y (some observations missing early on) [will take about 10 minutes]
% spl = [2004 7; 2019 12];    % Use for SOV_BONDS_RATE_PESOS_10Y  [will take about 10 minutes]
% spl = [2007 11; 2019 12];    % Use for BANK_BOND_SPREAD_2Y_TO_5Y_AAA
% spl = [2005 1; 2019 12];    % Use for BANK_BOND_SPREAD_2Y_TO_5Y_AA
spl = [2004 12; 2019 12];    % Use for ORPORATE_BOND_SPREAD_7Y_TO_10Y_AA

modname = 'ch1';            % ch1 is the 5 variable VAR with S_BBG and IRFs are computed for S_BBG shocks using Cholesky
                            % ch2 is the 4 variable VAR and IRFs are computed for MP_Rate shocks using Cholesky
                            % ch3 is the 5 variable VAR with Sh_CHSWPC and IRFs are computed for Sh_CHSWPC shocks using Cholesky
                            % ch4 is the 5 variable VAR with Sh_CHSWP2 and IRFs are computed for Sh_CHSWP2 shocks using Cholesky

addvar = 'CORP_BOND_SPR_7Y_TO_10Y_AA';                % We add additional variables (one at a time) by adding their names here 

% BANK_BOND_SPREAD_2Y_TO_5Y_AA
% BANK_BOND_SPREAD_2Y_TO_5Y_AAA
% CORPORATE_BOND_SPREAD_7Y_TO_10Y_A
% CORPORATE_BOND_SPREAD_7Y_TO_10Y_AA
% EEE_INFLATION_IN_11_MONTHS
% EOF_INFLATION_12_MONTHS
% LOG_COPPER_PRICE
% LOG_MINING
% SOV_BONDS_RATE_PESOS_10Y
% SOV_BONDS_RATE_PESOS_1Y
% SOV_BONDS_RATE_PESOS_2Y
% SOV_BONDS_RATE_PESOS_20Y
% SOV_BONDS_RATE_PESOS_5Y

% IRF Options 

MAlags = 120;                % Horizon for the IRF
                            
%%%%%%%%%% End of Options %%%%%%%%%%

% IDENTIFICATION: idscheme, mnames

idscheme = 'chol';          % In all cases identification will be via Cholesky


% define y

switch modname
    case 'ch1'
        ynames = {'LOG_CPI','LOG_IMACEC','LOG_EXCHANGE_RATE','MP_RATE'};  
        ny1 = 5;   
        mnames = {'S_BBG'};         % Chilean data with surprises     
        ss = 1;                     % IRFs with respect to first shock
    case 'ch2'
        ynames = {'LOG_CPI','LOG_IMACEC','LOG_EXCHANGE_RATE','MP_RATE'};
        ny1 = 5;
        mnames = {};             % Chilean data without surprises (used for
                                 % getting standard Cholesky for MP shocks)  
        ss = 4;                  % IRFs with respect to fourth shock
    case 'ch3'
        ynames = {'LOG_CPI','LOG_IMACEC','LOG_EXCHANGE_RATE','MP_RATE'};  
        ny1 = 5;   
        mnames = {'Sh_CHSWPC'};     % Chilean data with surprises     
        ss = 1;                     % IRFs with respect to first shock   
    case 'ch4'
        ynames = {'LOG_CPI','LOG_IMACEC','LOG_EXCHANGE_RATE','MP_RATE'};  
        ny1 = 5;   
        mnames = {'Sh_CHSWP2'};     % Chilean data with surprises     
        ss = 1;                     % IRFs with respect to first shock   
    otherwise
        disp(modname), error('unknown modname');
end

if ~isempty(addvar)
    try
        findstrings(addvar,ynames);
    catch
        ynames = [ynames {addvar}]; modname = [modname '_' addvar];
    end
end

% PRIOR
prior.lags = 12;
prior.minnesota.tightness = .2;
prior.minnesota.decay = 1;
prior.Nm = length(mnames);

% % create the output folder based on the calling filename
st = dbstack; pathout = st(end).name; clear st
mkdir(pathout)
pathout = [pathout '/'];

% functions for time operations
ym2t = @(x) x(1)+x(2)/12 - 1/24; % convert [year month] into time
t2datestr = @(t) [num2str(floor(t)) char(repmat(109,length(t),1)) num2str(12*(t-floor(t)+1/24),'%02.0f')];
t2ym = @(t) [floor(t) round(12*(t-floor(t)+1/24))]; % convert time into [year month]
ymdif = @(x1,x2) (x2(1)-x1(1))*12+x2(2)-x1(2);
findym = @(x,t) find(abs(t-ym2t(x))<1e-6); % find [year month] in time vector t

% Gibbs sampler settings
gssettings.ndraws = 4000;
gssettings.burnin = 4000;
gssettings.saveevery = 4;
gssettings.computemarglik = 0;

% Nice names for the mnames variable

mdict = { 'S_BBG', 'Bloomberg \newline Surprises ', 'Sh_CHSWPC', '3M Swap \ newline Surprises'};
mnames_nice = applydictregexp(mnames, mdict);
mylimits = nan(length(mnames),2);

% nice_names, yylimits, nonst

dictfname = 'data_var/ydict_chilean.csv';
fileID = fopen(dictfname);
ydict = textscan(fileID,'%s %q %d','Delimiter',',','HeaderLines',1);
fclose(fileID);
ynames_nice = applydict(ynames, [ydict{1} ydict{2}]);%ynames_nice = ynames;
% yylimits = [ydict{4} ydict{5}];       % This bit requires lb / ub in the dict file
% yylimits = yylimits(findstrings(ynames,ydict{1}),:);
nonst = ydict{3};
nonst = nonst(findstrings(ynames,ydict{1}),:);

% load data

datafname = 'data_var/data_chilean.csv';

data.Nm = length(mnames);
data.names = [mnames ynames];
d = importdata(datafname); dat = d.data; txt = d.textdata;
tbeg = find(dat(:,1)==spl(1,1) & dat(:,2)==spl(1,2)); if isempty(tbeg), tbeg=1; end
tend = find(dat(:,1)==spl(2,1) & dat(:,2)==spl(2,2)); if isempty(tend), tend=size(dat,1); end
ysel = findstrings(data.names, txt(1,:));
data.y = dat(tbeg:tend, ysel);
data.w = ones(size(data.y,1),1);
data.time = linspace(ym2t(dat(tbeg,1:2)), ym2t(dat(tend,1:2)), size(data.y,1))';
clear d dat txt tbeg tend ysel

% check ydata for missing values, determine the sample

if ~isempty(addvar)
    datatemp = checkdata(data, t2datestr, [1:data.Nm,size(data.y,2)]);   % Ignore if surprises or addvar is empty
else
    datatemp = checkdata(data, t2datestr, 1:data.Nm);
end
idspl = [t2datestr(datatemp.time(1)) '-' t2datestr(datatemp.time(end))];
clear datatemp

% % output file names
fname = [pathout modname '_' strjoin(mnames,'_') '_' idspl '_' idscheme];
diary([fname '.txt'])
if ~isempty(addvar)
    data = checkdata(data, t2datestr, [1:data.Nm,size(data.y,2)]);   % Ignore if surprises or addvar is empty
else
    data = checkdata(data, t2datestr, 1:data.Nm); % again, for the diary
end
plot_y;

% print the correlation matrix of m
% table = corr(data.y(:,1:data.Nm),'rows','pairwise');
% in.cnames = strvcat(mnames);
% in.rnames = strvcat(['correl.:' mnames]);
% in.width = 200;
% mprint(table,in)

% complete the minnesota prior
prior.minnesota.mvector = [zeros(data.Nm,1); nonst];

% replace NaNs with zeros in the initial condition
temp = data.y(1:prior.lags,:); temp(isnan(temp)) = 0; data.y(1:prior.lags,:) = temp; 

% estimate the VAR
res = VAR_withiid1kf(data, prior, gssettings);

savedata([fname '_data.csv'], data, t2ym)

%% identification

N = length(data.names);

switch idscheme
    case 'chol'
        shocknames = data.names;
        irfs_draws = NaN(N,N,MAlags,gssettings.ndraws);
        for i = 1:gssettings.ndraws
            betadraw = res.beta_draws(1:end-size(data.w,2),:,i);
            sigmadraw = res.sigma_draws(:,:,i);
            response = impulsdtrf(reshape(betadraw',N,N,prior.lags), chol(sigmadraw), MAlags);
            irfs_draws(:,:,:,i) = response;
        end
        
        if length(mnames)>1 && ((~isempty(strfind(mnames{1},'neg')) && ~isempty(strfind(mnames{2},'pos'))) || ~isempty(strfind(mnames{2},'_signrestr'))), ss = 1:2; end
end

%% reporting

% report variance decompositon
vdec_mean = table_vdecomp(irfs_draws, 1:N, ss, data.names, shocknames, 24);

% report the irfs:
qtoplot = [0.5 0.16 0.84 0.05 0.95]; % quantiles to plot
varnames = [mnames, ynames]; varnames_nice = [mnames_nice ynames_nice]; shocknames_nice = shocknames;
ylimits = [];
%ylimits = [mylimits; yylimits];
transf = nan(N,2);

% print out the impact responses
table_irf(irfs_draws, ss, 1, varnames, qtoplot(1:3));
table_irf(irfs_draws, ss, 1, varnames, qtoplot([1 4 5]));

% plot the irfs

hh = plot_irfs_draws(irfs_draws, data.Nm+1:min(N,data.Nm+ny1), ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [1 0 0], 'IRF_chol', ylimits, transf); saveTightFigure(hh,[fname '_irfy'],'pdf')

%hh = plot_irfs_draws(irfs_draws, data.Nm+1:min(N,data.Nm+ny1), ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [0 0 1], '', ylimits, transf); align_Ylabels(hh); saveTightFigure(hh,[fname '_irfy1'],'pdf')
%hh = plot_irfs_draws(irfs_draws, data.Nm+ny1+1:min(N,data.Nm+2*ny1), ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [0 0 1], '', ylimits); align_Ylabels(hh); saveTightFigure(hh,[fname '_irfy2'],'pdf')
%hh = plot_irfs_draws(irfs_draws, 1:N, ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [0 0 1], '', ylimits); align_Ylabels(hh); saveTightFigure(hh,[fname '_irfmy'],'pdf')

if 0 % save the ylimits
    hh = plot_irfs_draws(irfs_draws, 1:N, ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [0 0 1], '', ylimits, transf); align_Ylabels(hh);
    ylimits = cell2mat(get(hh.Children,'Ylim')); ylimits = ylimits(1:max(ss):N*max(ss),:); ylimits = flipud(ylimits);
    save([fname '_ylimits.mat'],'ylimits');
end

if ~isempty(addvar)
    ylimits = [];
%     ylimits = [mylimits; yylimits];
    varstoplot = findstrings(addvar,varnames);
    hh = plot_irfs_draws(irfs_draws, varstoplot, ss, varnames_nice, varnames, shocknames_nice, idscheme, qtoplot, [0 0 1], '', ylimits);
    saveTightFigure(hh,[fname '_addvar'],'pdf')
end

% if exist('irfs_l_draws','var')
% credibility = [0.68 0.9];
% hh = plot_irfs_draws_robust(irfs_draws, irfs_l_draws, irfs_u_draws, data.Nm+1:min(N,data.Nm+ny1), ss, varnames_nice, shocknames_nice, credibility, [1 1 1]); align_Ylabels(hh); saveTightFigure(hh,[fname '_rirfy1'],'pdf')
% hh = plot_irfs_draws_robust(irfs_draws, irfs_l_draws, irfs_u_draws, data.Nm+ny1+1:N, ss, varnames_nice, shocknames_nice, credibility, [0 0 1]); align_Ylabels(hh); saveTightFigure(hh,[fname '_rirfy2'],'pdf')
% %hh = plot_irfs_draws_robust(irfs_draws, irfs_l_draws, irfs_u_draws, 5:6, ss, varnames_nice, shocknames_nice, idscheme, credibility, [1 0 1]); align_Ylabels(hh); saveTightFigure(hh,[fname '_rirfyipcpi'],'pdf')
% end

diary off

