%[hx,ETA,R2,A,B,stdu,u] = svar_estim(a,R);
%estimate the matrices hx and ETA defining  the  structural vector autoregressive system
%x_t = hx x_t-1 + ETA e_t
%The identification assumption is that the matrix ETA is lower triangular. The innovation e_t~(0,I); 
%The data are contained  in the matrix a. Each row of a is an observation and each column is  a regressor.
%With Cholesky then hx and ETA are obtained  by OLS estimation (equation by equation) of  the system
%x_t = hx  x_t-1 + u_t
%and then  calculating ETA as the lower Cholesky decomposition of the variance/covariance matrix of the estimated residuals u_t.
%The matrix R introduces zero restrictions on the coefficients of hx (B) if 
%cholesky=1. Spcifically, if R(i,j)=0 and cholesky=1, 
%then hx(i,j) is restricted to be 0, whereas if R(i,j)=1, then 
%hx(i,j) is unrestricted. The default is R(i,j)=1 for all i,j  (use [] to invoke the default).  
%R2 is a vector of R-squares for each equation. 
%stdu is a vector of standard deviations of u_t
%u is a matrix such that u(t,:)=u_t'
%The two estimation methods (cholesky=1,0) give identical results when the system is unrestricted (all entries of R equal to 1), but not necessarily when the system has some zero restrictions. 
%This program can be used to estimate higher-order SVAR systems, since a higher-order system can always be reduced to a first-order system by an appropriate redefinition of variables. 
% Pablo Muoz Mayo 2020 using reduced version of Martn Uribe, May 2016.
function [hx,ETA,R2,A,B,stdu,u,b] = svar_estim_PM(a,R,lags);

if nargin<2|isempty(R)
R = ones(size(a,2));
end

nq = size(a,2); %number of equations in the svar system
a1 = lagg_BA(a,lags);

D = a1(:,1:lags*nq); %data [1:2] + lagged data (1) [3:4] + lagged data (2) [5:6] + lagged data (3) [7:8] + lagged data (4) [9:10] + lagged data (5) [11:12] + lagged data (6) [13:14]
D1 = a1(:,1+nq:(lags+1)*nq); % lagged data (1) [3:4] + lagged data (2) [5:6] + lagged data (3) [7:8] + lagged data (4) [9:10] + lagged data (5) [11:12] + lagged data (6) [13:14] + lagged data (7) [15:16]

T = size(D,1); %time length
cons = ones(T,1); %constant 

hx = zeros(nq*lags);
ETA = zeros(nq*lags,2);
A = [];
B = [];
u = zeros(T,nq*lags);

for i=1:nq
r = find(R(i,:)==1);
x = [D1(:,r) cons]; %independent variables
y = D(:,i); %dependent variable
b = x\y; %OLS regression coefficients
u(:,i) =  y - x*b; %estimated residuals
hx(i,r) = b(1:end-1);
end
ETA = chol(cov(u),'lower');

R2 = (1-var(u)./var(D))'; %Vector of R-square statistics
stdu = std(u);